// Array global para almacenar los jugadores seleccionados y los maximos globales de sus estadisticas
let jugadoresSeleccionados = [];
let maximosGlobales = {};

//Ejecuta al iniciar la pagina web el apartado de Datos de temporada
document.addEventListener("DOMContentLoaded", () => {
    restaurarFormaOriginal();
    datosTemporada();
    cargarDatosTemporada();
    cargarGraficosTemporada();
    resaltarBotonActivo("#div-superior");
});

//Evento que abre el apartado de datos de temporada al clickar en su div
d3.select("#div-superior").on("click", () => {
    restaurarFormaOriginal();
    datosTemporada();
    cargarDatosTemporada();
    cargarGraficosTemporada();
    resaltarBotonActivo("#div-superior");
});

//Evento que abre el apartado de rendimiento de jugadores al clickar en su div
d3.select("#div-inferior").on("click", () => {
    restaurarFormaOriginal();
    rendimientoJugadores();
    cargarJugadoresDesdeArchivo();
    resaltarBotonActivo("#div-inferior");
});

// Función para restaurar el div #resto-pantalla a su forma original
function restaurarFormaOriginal() {
    d3.select("#resto-pantalla")
        .style("display", "flex")
        .style("flex-direction", "row")
        .selectAll(".sector")
        .remove();
}

// Función para resaltar el div(boton) del apartado que se esta visualizando
function resaltarBotonActivo(selector) {
    d3.selectAll("#div-superior, #div-inferior").style("background-color", "#B3E5FC").style("color", "#333");
    d3.select(selector).style("background-color", "#0288D1").style("color", "#FFF").style("font-weight", "bold");
}


//Funciones del apartado "Datos temporada"

//Modifica el div resto de pantalla para el apartado "Datos de temporada" añadiendole cuatro divs con sus titulos
function datosTemporada() {
    const restoPantalla = d3.select("#resto-pantalla");

    restoPantalla.selectAll("*").remove();

    restoPantalla
        .style("display", "grid")
        .style("grid-template-columns", "1fr 1fr")
        .style("grid-template-rows", "1fr 1fr")
        .style("gap", "0");

    const data = [
        { title: "Victorias/Derrotas", content: "" },
        { title: "Número total de puntos", content: "" },
        { title: "Posición Final", content: "" },
        { title: "Porcentaje de tiro", content: "" },
    ];

    restoPantalla
        .selectAll(".sector")
        .data(data)
        .enter()
        .append("div")
        .attr("class", "sector")
        .style("background-color", "#4FC3F7")
        .style("flex-direction", "column")
        .style("align-items", "flex-start")
        .style("justify-content", "flex-start")
        .style("padding", "10px")
        .each(function (d) {
            d3.select(this)
                .append("div")
                .attr("class", "sector-title")
                .style("font-weight", "bold")
                .style("font-size", "1.2em")
                .style("margin-bottom", "10px")
                .text(d.title);

            d3.select(this)
                .append("div")
                .attr("class", "sector-content")
                .style("flex", "1")
                .text(d.content);
        });
}

//Carga los datos de la temporada actualizando el contenido de la PosicionFinal y el Número Total de Puntos
function cargarDatosTemporada() {
    const csvPath = "assets/estadisticasDatosTemporada.csv";

    // Leer el archivo CSV
    d3.csv(csvPath).then((data) => {
        console.log("Datos cargados del CSV:", data);

        // Ajustar las claves según las columnas del CSV
        const numeroTotalPuntos = parseInt(data[0]["Número total de puntos"], 10);
        const posicionFinal = data[0]["Clasificación"];

         // Actualizar el sector "Numero Total de Puntos"
        d3.select("#resto-pantalla")
            .selectAll(".sector")
            .filter((d) => d.title === "Número total de puntos")
            .select(".sector-content")
            .html(`<div class="animated-points" style="font-size: 4em; font-weight: bold;">${numeroTotalPuntos}</div>`);


        // Actualizar el sector "Posición Final"
        d3.select("#resto-pantalla")
            .selectAll(".sector")
            .filter((d) => d.title === "Posición Final")
            .select(".sector-content")
            .html(`<div style="text-align: center; font-size: 3em; font-weight: bold;">${posicionFinal}º</div>`);
    }).catch((error) => {
        console.error("Error cargando el archivo CSV:", error);
    });
}

//Carga los datos de la temporada actualizando el contenido de los sectores Victorias/Derrotas y Porcentaje de tiro
function cargarGraficosTemporada() {
    const csvPath = "assets/estadisticasDatosTemporada.csv";

    // Leer el archivo CSV
    d3.csv(csvPath).then((data) => {
        console.log("Datos cargados del CSV:", data);

       
        const victorias = parseInt(data[0]["Partidos ganados"], 10);
        const derrotas = parseInt(data[0]["Partidos perdidos"], 10);
        const porcentajeTiroDos = parseFloat(data[0]["% de tiros de dos"]);
        const porcentajeTiroTres = parseFloat(data[0]["% de tiros de tres"]);
        const porcentajeTiroLibres = parseFloat(data[0]["% de tiros libres"]);

        const totalPartidos = victorias + derrotas;
        const porcentajeVictorias = (victorias / totalPartidos) * 100;

        // Actualizar sector "Victorias/Derrotas" con gráfico de anillo
        const sectorVictorias = d3.select("#resto-pantalla .sector:nth-child(1) .sector-content");
        agregarGraficoAnillo(
            sectorVictorias.node(),
            porcentajeVictorias,
            ["#0288D1", "#B3E5FC"], // Colores
            `${porcentajeVictorias.toFixed(1)}%`
        );

        // Añadir texto informativo al lado del gráfico
        sectorVictorias.selectAll(".texto-informativo").remove();
        sectorVictorias.append("div")
            .attr("class", "texto-informativo")
            .style("margin", "40px")
            .style("text-align", "left")
            .style("font-size", "0.6em")
            .html(`
                - Partidos jugados: ${totalPartidos} <br><br>
                - Partidos ganados: ${victorias} <br><br>
                - Partidos perdidos: ${derrotas}
            `);

        // Actualizar sector "Porcentaje de tiro" con gráfico de anillo y checkboxes
        const sectorPorcentajeTiro = d3.select("#resto-pantalla .sector:nth-child(4) .sector-content");

        // Contenedor para el gráfico inicial
        agregarGraficoAnillo(
            sectorPorcentajeTiro.node(),
            porcentajeTiroDos,
            ["#0288D1", "#B3E5FC"],
            `${porcentajeTiroDos.toFixed(1)}%`
        );

        // Contenedor para checkboxes
        let checkboxesContainer = sectorPorcentajeTiro.select(".checkboxes-tiro");
        if (checkboxesContainer.empty()) {
            checkboxesContainer = sectorPorcentajeTiro.append("div")
                .attr("class", "checkboxes-tiro")
                .style("margin", "40px")
                .style("display", "flex")
                .style("flex-direction", "column")
                .style("align-items", "flex-start")
                .style("font-size", "0.6em");
        }

        checkboxesContainer.selectAll("*").remove(); // Limpiar checkboxes previos

        ["% de tiros de dos", "% de tiros de tres", "% de tiros libres"].forEach((label, index) => {
            const checkbox = checkboxesContainer.append("label")
                .style("display", "flex")
                .style("align-items", "center")
                .style("margin-bottom", "10px");

            const input = checkbox.append("input")
                .attr("type", "radio")
                .attr("name", "tipo-tiro")
                .attr("value", index === 0 ? porcentajeTiroDos : index === 1 ? porcentajeTiroTres : porcentajeTiroLibres)
                .style("margin-right", "10px")
                .on("change", function () {
                    const selectedValue = parseFloat(this.value);
                    agregarGraficoAnillo(
                        sectorPorcentajeTiro.node(),
                        selectedValue,
                        ["#0288D1", "#B3E5FC"],
                        `${selectedValue.toFixed(1)}%`
                    );
                });

            // Marcar el checkbox de "% de tiros de dos" como seleccionado por defecto
            if (index === 0) {
                input.attr("checked", true);
            }

            checkbox.append("span").text(label);
        });
    }).catch((error) => {
        console.error("Error cargando el archivo CSV:", error);
    });
}

// Función que crea loa graficos anillos de los sectores victorias/derrotas y Porcentaje tiro
function agregarGraficoAnillo(containerSelector, porcentaje, colores, textoInterno) {
    const container = d3.select(containerSelector);

    let graficoContainer = container.select(".grafico-anillo");
    if (graficoContainer.empty()) {
        graficoContainer = container.append("div").attr("class", "grafico-anillo");
    }

    graficoContainer.selectAll("*").remove(); 

    const width = 350; 
    const height = 350;
    const radius = Math.min(width, height) / 2;

    const svg = graficoContainer
        .append("svg")
        .attr("width", width)
        .attr("height", height)
        .append("g")
        .attr("transform", `translate(${width / 2}, ${height / 2})`);

   
    const arc = d3.arc()
        .innerRadius(radius * 0.7) 
        .outerRadius(radius) 
        .startAngle(0)
        .endAngle((2 * Math.PI) * (porcentaje / 100)); 

    // Dibuja la sección "rellena"
    svg.append("path")
        .attr("d", arc)
        .style("fill", colores[0]); 

    // Dibuja la sección "restante"
    svg.append("path")
        .attr("d", d3.arc()
            .innerRadius(radius * 0.7)
            .outerRadius(radius)
            .startAngle((2 * Math.PI) * (porcentaje / 100))
            .endAngle(2 * Math.PI))
        .style("fill", colores[1]); 

    // Agregar texto en el centro
    svg.append("text")
        .attr("text-anchor", "middle")
        .attr("dy", ".35em")
        .style("font-size", "0.8em") 
        .style("font-weight", "bold")
        .text(textoInterno);
}


//Funciones del apartado "Rendimiento Jugadores"

/*Modifica el div resto de pantalla para el apartado "RendimientoJugadores" añadiendole dos divs:
 lado-izquierdo(Para almacenar la lista de los jugadores)
 lado-derecho(Para mostrar el grafico radar)
*/
function rendimientoJugadores() {
    const restoPantalla = d3.select("#resto-pantalla");

    restoPantalla.selectAll("*").remove();

    restoPantalla
        .append("div")
        .attr("id", "lado-izquierdo");

    restoPantalla
        .append("div")
        .attr("id", "lado-derecho")
        .style("flex", "1")
        .style("background-color", "#F5F5F5")
        .style("align-items", "center")
        .style("justify-content", "center")
        .style("position", "relative");

    crearGraficoRadar("#lado-derecho");
}

// Calcular los máximos para cada estadística de los jugadores
function calcularMaximos(datos) {
    return {
        "Puntos/Partido": d3.max(datos, (d) => +d["Puntos/Partido"]),
        "T.2%": d3.max(datos, (d) => +d["T.2%"]),
        "T.3%": d3.max(datos, (d) => +d["T.3%"]),
        "T.L%": d3.max(datos, (d) => +d["T.L%"]),
        "Rebotes/Partido": d3.max(datos, (d) => +d["Rebotes/Partido"]),
        "Recuperaciones/Partido": d3.max(datos, (d) => +d["Recuperaciones/Partido"]),
        "Asistencias/Partido": d3.max(datos, (d) => +d["Asistencias/Partido"]),
        "Minutos/Partido": d3.max(datos, (d) => +d["Minutos/Partido"]),
    };
}


// Funcion que crea el grafico radar 
function crearGraficoRadar(containerSelector, jugadoresSeleccionados = [], maximos = null) {
    const container = d3.select(containerSelector);

    container.selectAll("*").remove();

    const boundingRect = container.node().getBoundingClientRect();
    const width = boundingRect.width;
    const height = boundingRect.height;
    const margin = 50;

    const metrics = 8;
    const levels = 5;
    const radius = (Math.min(width, height) - margin * 2) / 2 * 0.70;
    const angleSlice = (2 * Math.PI) / metrics;

    const labels = [
        "Puntos/Partido",
        "T.2%",
        "T.3%",
        "T.L%",
        "Rebotes/Partido",
        "Recuperaciones/Partido",
        "Asistencias/Partido",
        "Minutos/Partido",
    ];

    const rScale = d3.scaleLinear().range([0, radius]).domain([0, 100]);

    const svg = container
        .append("svg")
        .attr("width", "100%")
        .attr("height", "100%")
        .append("g")
        .attr("transform", `translate(${width / 2}, ${height / 2})`);

    for (let level = 1; level <= levels; level++) {
        const levelRadius = (level / levels) * radius;

        const vertices = [];
        for (let i = 0; i < metrics; i++) {
            vertices.push([
                levelRadius * Math.cos(angleSlice * i - Math.PI / 2),
                levelRadius * Math.sin(angleSlice * i - Math.PI / 2),
            ]);
        }

        for (let i = 0; i < vertices.length; i++) {
            const [x1, y1] = vertices[i];
            const [x2, y2] = vertices[(i + 1) % vertices.length];
            svg.append("line")
                .attr("x1", x1)
                .attr("y1", y1)
                .attr("x2", x2)
                .attr("y2", y2)
                .style("stroke", level === levels ? "#0288D1" : "#999")
                .style("stroke-width", level === levels ? 2 : 1);
        }
    }

    // Dibujar líneas radiales
    for (let i = 0; i < metrics; i++) {
        const x = radius * Math.cos(angleSlice * i - Math.PI / 2);
        const y = radius * Math.sin(angleSlice * i - Math.PI / 2);

        svg.append("line")
            .attr("x1", 0)
            .attr("y1", 0)
            .attr("x2", x)
            .attr("y2", y)
            .style("stroke", "#444")
            .style("stroke-width", 1.5);

        const offset = 1.15;
        const anchor = i === 0 || i === 4 ? "middle" : x > 0 ? "start" : "end";
        const dy = i === 0 ? "-0.5em" : i === 4 ? "0.9em" : "0.35em";

        svg.append("text")
            .attr("x", x * offset)
            .attr("y", y * offset)
            .attr("dy", dy)
            .style("text-anchor", anchor)
            .style("font-size", "14px")
            .style("font-weight", "bold")
            .style("fill", "#333")
            .text(labels[i]);
    }

    // Dibujar formas geométricas de los jugadores seleccionados
    jugadoresSeleccionados.forEach((jugador) => {
        const playerData = labels.map((label) => {
            const value = +jugador[label];
            const maximo = maximos[label]; // Usar los máximos globales
            return (value / maximo) * 100; // Escalar al porcentaje del máximo
        });

        const pathData = playerData.map((d, i) => {
            const r = rScale(d);
            const x = r * Math.cos(angleSlice * i - Math.PI / 2);
            const y = r * Math.sin(angleSlice * i - Math.PI / 2);
            return [x, y];
        });

        const polygonPath = d3.line()
            .x((d) => d[0])
            .y((d) => d[1])
            .curve(d3.curveLinearClosed);

        svg.append("path")
            .attr("d", polygonPath(pathData))
            .style("fill", jugador.color)
            .style("fill-opacity", 0.4)
            .style("stroke", jugador.color)
            .style("stroke-width", 2);
    });

    // Añadir los nombres de los jugadores seleccionados con sus colores en la esquina superior derecha
    const legendGroup = svg.append("g")
        .attr("transform", `translate(${radius + margin}, ${-radius - margin})`)
        .attr("class", "legend");

    jugadoresSeleccionados.forEach((jugador, index) => {
        const legendItem = legendGroup.append("g")
            .attr("transform", `translate(0, ${index * 20})`);

        // Círculo de color
        legendItem.append("circle")
            .attr("r", 8)
            .attr("fill", jugador.color);

        // Nombre del jugador
        legendItem.append("text")
            .attr("x", 15)
            .attr("y", 4)
            .style("font-size", "14px")
            .style("fill", "#333")
            .text(jugador.Nombre);
    });
}

//Funcion que carga las estadisticas de los jugadores
function cargarJugadoresDesdeArchivo() {
    const csvPath = "assets/estadisticasRendimientoJugadores.csv";
    const ladoIzquierdo = d3.select("#lado-izquierdo");

    ladoIzquierdo.selectAll("*").remove();

    ladoIzquierdo
        .style("display", "flex")
        .style("flex-direction", "column")
        .style("justify-content", "space-between");

    d3.csv(csvPath).then((data) => {
        // Calcula los máximos al cargar los datos
        maximosGlobales = calcularMaximos(data);

        const colores = [
            "#FF5733", "#33FF57", "#3357FF", "#F333FF", "#FFD733", "#33FFF5",
            "#FF33A1", "#A133FF", "#FF6F33", "#33FF94", "#FF3333", "#33A1FF",
            "#FF33D7", "#6FFF33", "#FF9333"
        ];

        // Asigna un color único a cada jugador
        data.forEach((d, i) => {
            d.color = colores[i % colores.length];
        });

        // Crear la lista de jugadores
        ladoIzquierdo
            .selectAll(".jugador-rect")
            .data(data)
            .enter()
            .append("div")
            .attr("class", "jugador-rect")
            .style("flex", "1")
            .style("margin", "2px 0")
            .style("padding", "10px")
            .style("background-color", "#B3E5FC")
            .style("color", "#000")
            .style("display", "flex")
            .style("align-items", "center")
            .style("justify-content", "center")
            .style("border", "2px solid transparent")
            .on("click", function (event, d) {
                const checkbox = d3.select(this).select(".jugador-checkbox");
                const isChecked = checkbox.property("checked");

                if (isChecked) {
                    checkbox.property("checked", false);
                    d3.select(this).style("border", "2px solid transparent");
                    jugadoresSeleccionados = jugadoresSeleccionados.filter((jugador) => jugador !== d);
                } else {
                    if (jugadoresSeleccionados.length === 2) {
                        const jugadorAntiguo = jugadoresSeleccionados.shift();
                        d3.selectAll(".jugador-checkbox")
                            .filter((data) => data === jugadorAntiguo)
                            .property("checked", false);
                        d3.selectAll(".jugador-rect")
                            .filter((data) => data === jugadorAntiguo)
                            .style("border", "2px solid transparent");
                    }

                    checkbox.property("checked", true);
                    d3.select(this).style("border", `3px solid ${d.color}`);
                    jugadoresSeleccionados.push(d);
                }

                // Usa los máximos globales al actualizar el gráfico
                crearGraficoRadar("#lado-derecho", jugadoresSeleccionados, maximosGlobales);
            })
            .each(function (d) {
                d3.select(this)
                    .append("div")
                    .attr("class", "jugador-numero")
                    .style("font-weight", "bold")
                    .style("font-size", "1.2em")
                    .style("flex", "0 0 auto")
                    .style("text-align", "left")
                    .text(`${d.Nº}º`);

                d3.select(this)
                    .append("div")
                    .attr("class", "jugador-nombre")
                    .style("font-size", "1.2em")
                    .style("text-align", "center")
                    .style("flex", "1")
                    .text(d.Nombre);

                d3.select(this)
                    .append("input")
                    .attr("type", "checkbox")
                    .attr("class", "jugador-checkbox")
                    .style("display", "none");
            });
    });
}

//Funcion que gestiona los jugadores selecionados y actualiza el grafico radar en base a estos
function gestionarCheckboxes(data, maximos) {
    const jugadoresSeleccionados = [];

    // Recopila los jugadores seleccionados actualmente
    d3.selectAll(".jugador-checkbox").each(function (d, i) {
        if (d3.select(this).property("checked")) {
            jugadoresSeleccionados.push(data[i]);
        }
    });

    // Si hay más de dos seleccionados, deselecciona el jugador más antiguo
    if (jugadoresSeleccionados.length > 2) {
        const jugadorAntiguo = jugadoresSeleccionados.shift(); // Elimina el más antiguo
        d3.selectAll(".jugador-checkbox")
            .filter((d) => d === jugadorAntiguo)
            .property("checked", false); // Desmarca el checkbox
        d3.selectAll(".jugador-rect")
            .filter((d) => d === jugadorAntiguo)
            .style("border", "2px solid transparent"); // Quita el borde
    }

    // Actualiza el gráfico de radar
    crearGraficoRadar("#lado-derecho", jugadoresSeleccionados, maximos);
}